# This file is part of librefdatool. librefdatool is free software: you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Copyright (C) 2013 Javier D. Garcia-Lasheras
#

import sys
import os

from simcore import simcore


class core:
    '''This class provides the principal librefdatool methods,
    including running signal filtering with HDL models.
    TBD: limited to FIR filter simulation
    '''


    def librefilter(self, b, a, x):
        '''This method provides filter hardware simulation,
        using a HDL description & a simulation engine 
        parametrized with the device control values.
        * c: FIR filter coefficients array.
        * x: Input signal array.
        * return: simulated output signal
        NOTE: the filter behaviour is tuned to match with:
        scipy.signal.lfilter 
        (the purpose is direct comparision of outputs)
        '''

        # Save path and jump to workspace (TBD: if exists!!)
        self.savedPath = os.getcwd()
        os.chdir(self.workspace)
        
        runner = simcore();
        
        response = runner.simfilter(b, a, x,
                self.structure, self.model, self.trace,
                self.busX, self.busY, self.busC, 
                self.scalingX, self.scalingC)



        # Return from workspace folder
        os.chdir(self.savedPath)

        return response




